@@ -28,20 +28,27 @@ package Win32::Setupsup;
 # it's only intended to work on winnt (version 4.0 with sp3 or later)
 # but it should work on win95/98/me too
 
-require Exporter;
-require DynaLoader;
 use 5.006;
 use strict;
 use warnings;
 
-use Win32::Registry;
+use Carp 'croak';
+use Exporter 'import';
+use Win32API::Registry qw(RegCloseKey RegCreateKeyEx RegOpenKeyEx
+                          RegQueryValueEx RegSetValueEx regLastError
+                          :KEY_ :HKEY_ :REG_);
+use XSLoader ();
 
-our $VERSION = '1.02';
-# This file is part of Win32-Setupsup 1.02 (November 5, 2011)
+our $VERSION = '1.03';
+# This file is part of Win32-Setupsup 1.03 (November 11, 2011)
 
-die "The Win32::Setupsup module works only on Windows NT" if(!Win32::IsWinNT());
+croak("The Win32::Setupsup module works only on Windows NT")
+    unless Win32::IsWinNT();
 
-our @ISA= qw(Exporter DynaLoader);
+# Aid porting from Win32::Registry:
+sub NULL () { [] }
+sub DWORD_1 () { "\x01\0\0\0" }
+sub DWORD_0 () { "\0\0\0\0" }
 
 # Items to export into caller's namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
@@ -182,8 +189,7 @@ sub AUTOLOAD
       $AutoLoader::AUTOLOAD = $AUTOLOAD;
       goto &AutoLoader::AUTOLOAD;
     } else {
-      my ($file,$line) = (caller)[1,2];
-      die "Your vendor has not defined Win32::Setupsup macro $constname, used in $file at line $line.";
+      croak "Your vendor has not defined Win32::Setupsup macro $constname";
     }
   }
   eval "sub $AUTOLOAD { $val }";
@@ -194,25 +200,24 @@ sub AUTOLOAD
 # disables keyboard input after reboot
 sub DisableKeyboardAfterReboot
 {
-  die "Usage: Win32::Setupsup::DisableKeyboardAfterReboot()\n" if($#_ != -1);
+  croak "Usage: Win32::Setupsup::DisableKeyboardAfterReboot()" if @_;
 
   my ($hKey, $disp);
-  if (!Win32::Registry::RegCreateKeyEx( &HKEY_LOCAL_MACHINE,
+  if (!RegCreateKeyEx( HKEY_LOCAL_MACHINE,
         'SYSTEM\\CurrentControlSet\\Hardware Profiles\\' .
         '0001\\System\\CurrentControlSet\\Enum\\ROOT\\LEGACY_KBDCLASS\\0000',
-        &NULL, '', &NULL, &KEY_WRITE, &NULL, $hKey, $disp)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
+        NULL, '', NULL, KEY_WRITE, NULL, $hKey, $disp)) {
+    Win32::Setupsup::SetLastError(regLastError());
     return 0;
   }
 
-  if (!Win32::Registry::RegSetValueEx($hKey, 'CSConfigFlags', &NULL,
-                                      &REG_DWORD, 1)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
-    Win32::Registry::RegCloseKey($hKey);
+  if (!RegSetValueEx($hKey, 'CSConfigFlags', NULL, REG_DWORD, DWORD_1)) {
+    Win32::Setupsup::SetLastError(regLastError());
+    RegCloseKey($hKey);
     return 0;
   }
 
-  Win32::Registry::RegCloseKey($hKey);
+  RegCloseKey($hKey);
 
   return 1;
 }
@@ -221,25 +226,24 @@ sub DisableKeyboardAfterReboot
 # enables keyboard input after reboot
 sub EnableKeyboardAfterReboot
 {
-  die "Usage: Win32::Setupsup::EnableKeyboardAfterReboot()\n" if($#_ != -1);
+  croak "Usage: Win32::Setupsup::EnableKeyboardAfterReboot()" if @_;
 
   my ($hKey, $disp);
-  if (!Win32::Registry::RegCreateKeyEx(&HKEY_LOCAL_MACHINE,
+  if (!RegCreateKeyEx(HKEY_LOCAL_MACHINE,
         'SYSTEM\\CurrentControlSet\\Hardware Profiles\\' .
         '0001\\System\\CurrentControlSet\\Enum\\ROOT\\LEGACY_KBDCLASS\\0000',
-        &NULL, '', &NULL, &KEY_WRITE, &NULL, $hKey, $disp)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
+        NULL, '', NULL, KEY_WRITE, NULL, $hKey, $disp)) {
+    Win32::Setupsup::SetLastError(regLastError());
     return 0;
   }
 
-  if (!Win32::Registry::RegSetValueEx($hKey, 'CSConfigFlags', &NULL,
-                                      &REG_DWORD, 0)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
-    Win32::Registry::RegCloseKey($hKey);
+  if (!RegSetValueEx($hKey, 'CSConfigFlags', NULL, REG_DWORD, DWORD_0)) {
+    Win32::Setupsup::SetLastError(regLastError());
+    RegCloseKey($hKey);
     return 0;
   }
 
-  Win32::Registry::RegCloseKey($hKey);
+  RegCloseKey($hKey);
 
   return 1;
 }
@@ -248,25 +252,24 @@ sub EnableKeyboardAfterReboot
 # disables mouse input after reboot
 sub DisableMouseAfterReboot
 {
-  die "Usage: Win32::Setupsup::DisableMouseAfterReboot()\n" if($#_ != -1);
+  croak "Usage: Win32::Setupsup::DisableMouseAfterReboot()" if @_;
 
   my ($hKey, $disp);
-  if (!Win32::Registry::RegCreateKeyEx(&HKEY_LOCAL_MACHINE,
+  if (!RegCreateKeyEx(HKEY_LOCAL_MACHINE,
         'SYSTEM\\CurrentControlSet\\Hardware Profiles\\' .
         '0001\\System\\CurrentControlSet\\Enum\\ROOT\\LEGACY_MOUCLASS\\0000',
-        &NULL, '', &NULL, &KEY_WRITE, &NULL, $hKey, $disp)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
+        NULL, '', NULL, KEY_WRITE, NULL, $hKey, $disp)) {
+    Win32::Setupsup::SetLastError(regLastError());
     return 0;
   }
 
-  if (!Win32::Registry::RegSetValueEx($hKey, 'CSConfigFlags', &NULL,
-                                      &REG_DWORD, 1)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
-    Win32::Registry::RegCloseKey($hKey);
+  if (!RegSetValueEx($hKey, 'CSConfigFlags', NULL, REG_DWORD, DWORD_1)) {
+    Win32::Setupsup::SetLastError(regLastError());
+    RegCloseKey($hKey);
     return 0;
   }
 
-  Win32::Registry::RegCloseKey($hKey);
+  RegCloseKey($hKey);
 
   return 1;
 }
@@ -275,25 +278,24 @@ sub DisableMouseAfterReboot
 # enables mouse input after reboot
 sub EnableMouseAfterReboot
 {
-  die "Usage: Win32::Setupsup::EnableMouseAfterReboot()\n" if($#_ != -1);
+  croak "Usage: Win32::Setupsup::EnableMouseAfterReboot()" if @_;
 
   my ($hKey, $disp);
-  if (!Win32::Registry::RegCreateKeyEx(&HKEY_LOCAL_MACHINE,
+  if (!RegCreateKeyEx(HKEY_LOCAL_MACHINE,
         'SYSTEM\\CurrentControlSet\\Hardware Profiles\\' .
         '0001\\System\\CurrentControlSet\\Enum\\ROOT\\LEGACY_MOUCLASS\\0000',
-        &NULL, '', &NULL, &KEY_WRITE, &NULL, $hKey, $disp)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
+        NULL, '', NULL, KEY_WRITE, NULL, $hKey, $disp)) {
+    Win32::Setupsup::SetLastError(regLastError());
     return 0;
   }
 
-  if (!Win32::Registry::RegSetValueEx($hKey, 'CSConfigFlags', &NULL,
-                                      &REG_DWORD, 0)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
-    Win32::Registry::RegCloseKey($hKey);
+  if (!RegSetValueEx($hKey, 'CSConfigFlags', NULL, REG_DWORD, DWORD_0)) {
+    Win32::Setupsup::SetLastError(regLastError());
+    RegCloseKey($hKey);
     return 0;
   }
 
-  Win32::Registry::RegCloseKey($hKey);
+  RegCloseKey($hKey);
 
   return 1;
 }
@@ -302,23 +304,22 @@ sub EnableMouseAfterReboot
 # gets the program files directory from registry
 sub GetProgramFilesDir
 {
-  die "Usage: Win32::Setupsup::GetProgramFilesDir(\\\$dir)\n" if($#_);
+  croak "Usage: Win32::Setupsup::GetProgramFilesDir(\\\$dir)" if ($#_);
 
   my $hKey;
-  if (!Win32::Registry::RegOpenKeyEx(&HKEY_LOCAL_MACHINE,
-        'Software\Microsoft\Windows\CurrentVersion', &NULL, &KEY_READ, $hKey)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
+  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+        'Software\Microsoft\Windows\CurrentVersion', NULL, KEY_READ, $hKey)) {
+    Win32::Setupsup::SetLastError(regLastError());
     return 0;
   }
 
-  if (!Win32::Registry::RegQueryValueEx($hKey, 'ProgramFilesDir', &NULL,
-                                        &NULL, $_[0])) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
-    Win32::Registry::RegCloseKey($hKey);
+  if (!RegQueryValueEx($hKey, 'ProgramFilesDir', NULL, NULL, $_[0])) {
+    Win32::Setupsup::SetLastError(regLastError());
+    RegCloseKey($hKey);
     return 0;
   }
 
-  Win32::Registry::RegCloseKey($hKey);
+  RegCloseKey($hKey);
 
   return 1;
 }
@@ -327,29 +328,28 @@ sub GetProgramFilesDir
 # gets the common files directory from registry
 sub GetCommonFilesDir
 {
-  die "Usage: Win32::Setupsup::GetCommonFilesDir(\\\$dir)\n" if($#_);
+  croak "Usage: Win32::Setupsup::GetCommonFilesDir(\\\$dir)\n" if($#_);
 
   my $hKey;
-  if (!Win32::Registry::RegOpenKeyEx(&HKEY_LOCAL_MACHINE,
-        'Software\Microsoft\Windows\CurrentVersion', &NULL, &KEY_READ, $hKey)) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
+  if (!RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+        'Software\Microsoft\Windows\CurrentVersion', NULL, KEY_READ, $hKey)) {
+    Win32::Setupsup::SetLastError(regLastError());
     return 0;
   }
 
-  if (!Win32::Registry::RegQueryValueEx($hKey, 'CommonFilesDir', &NULL,
-                                        &NULL, $_[0])) {
-    Win32::Setupsup::SetLastError(Win32::GetLastError());
-    Win32::Registry::RegCloseKey($hKey);
+  if (!RegQueryValueEx($hKey, 'CommonFilesDir', NULL, NULL, $_[0])) {
+    Win32::Setupsup::SetLastError(regLastError());
+    RegCloseKey($hKey);
     return 0;
   }
 
-  Win32::Registry::RegCloseKey($hKey);
+  RegCloseKey($hKey);
 
   return 1;
 }
 
 
-__PACKAGE__->bootstrap;
+XSLoader::load(__PACKAGE__, $VERSION);
 
 1;
 
@@ -361,12 +361,12 @@ Win32::Setupsup - Remote control for Windows applications
 
 =head1 VERSION
 
-This document describes version 1.02 of
-Win32::Setupsup, released November 5, 2011
+This document describes version 1.03 of
+Win32::Setupsup, released November 11, 2011
 
 =head1 SYNOPSIS
 
-        use Win32::Setupsup;
+  use Win32::Setupsup;
 
 =head1 DESCRIPTION
 
@@ -458,6 +458,8 @@ GetProgramFilesDir
 GetThreadLastError
 SetLastError
 constant
+^DWORD_\d$
+^NULL$
 
 =over
 
